/*
 * Copyright (C) 2009 - 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include "Common.h"
#include "treemanager/IMOTreeManager.h"
#include <syncml/core/Item.h>

namespace NS_DM_Client
{
	bool Serialize(const String& path, const String& data, IMOTreeManager& treeManager, const String& format /*= "chr"*/)
	{
		Funambol::Item item;
		Funambol::Target target(path.c_str());
		item.setTarget(&target);
		Funambol::Meta meta;
		meta.setType("text/plain");
		meta.setFormat(format.c_str());
		item.setMeta(&meta);
		Funambol::ComplexData complexData(data.c_str());
		item.setData(&complexData);

		StatusCode status = treeManager.Add(path, item, 0, false);
 		if (e_AlreadyExists == status)
 		{
 			status = treeManager.Replace(path, item, 0, false);
 		}
		return status == e_Ok? true: false;
	}
	//-------------------------------------------------------------------------------------------

	bool Deserialize(const String& path, String& data, IMOTreeManager& treeManager)
	{
		return (e_Ok == treeManager.GetValue(path, data));
	}
	//-------------------------------------------------------------------------------------------

	bool Deserialize(const String& path, size_t& data, IMOTreeManager& treeManager)
	{
		String value;
		bool res = (e_Ok == treeManager.GetValue(path, value));
		if (res)
		{
			data = atoi(value.c_str());
		}
		return res;
	}
	//-------------------------------------------------------------------------------------------

	bool Deserialize(const String& path, int& data, IMOTreeManager& treeManager)
	{
		String value;
		bool res = (e_Ok == treeManager.GetValue(path, value));
		if (res)
		{
			data = atoi(value.c_str());
		}
		return res;
	}

	//-------------------------------------------------------------------------------------------

	bool Deserialize(const String& path, bool& data, IMOTreeManager& treeManager)
	{
		String value;
		bool res = (e_Ok == treeManager.GetValue(path, value));
		if (res)
		{
			data = (value == S_true);
		}
		return res;
	}
	//-------------------------------------------------------------------------------------------

	bool Deserialize(const String& path, UINT8& data, IMOTreeManager& treeManager)
	{
		String value;
		bool res = (e_Ok == treeManager.GetValue(path, value));
		if (res)
		{
			data = atoi(value.c_str());
		}
		return res;
	}
	//-------------------------------------------------------------------------------------------

	void ToString(String& destination, void* source, int size)
	{
		char* tmpContent = new char[size + 1];
		memset(tmpContent, 0, size + 1); // last 0 added
		memcpy(tmpContent, source, size);
		destination = tmpContent;
		delete [] tmpContent;
	}
	//-------------------------------------------------------------------------------------------

	String BoolToString(bool source)
	{
		String res = source? S_true: S_false;
		return res;
	}
}
