/*
 * Copyright (C) 2009 - 2010 Funambol, Inc.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 *
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

#include <boost/test/unit_test.hpp>
#include <executionqueue/ExecutionQueue.h>

#include "commontypes.h"
#include "daemon/ProfileComponentsHolder.h"

#include <treemanager/MOTreeCommand.h>
#include <executionqueue/ICommandDispatcher.h>

using namespace NS_DM_Client;
using namespace NS_DM_Client::NS_ExecutionQueue;


static NS_Logging::Logger& logger = NS_Logging::GetLogger("ExecutionQueue");

class SupportedMOTreeCommand : public MOTreeCommand
{
public:
    virtual bool Execute()
    {
        return true;
    }

};

class UnsupportedCommand : public ICommand
{
public:
    virtual bool Execute()
    {
        return true;
    }

    bool Visit(ICommandDispatcher& cmdDispatcher)
    {
        return cmdDispatcher.Dispatch(*this);
    }

};


BOOST_AUTO_TEST_CASE(AddSupportedCommand)
{
	LOG_DEBUG_(logger, "AddSupportedCommand test is running. ");

    IExecutionQueue& eq = *new ExecutionQueue;

    StringMap params;
    String loggerInstanceName = "ExecutionQueue";
    ProfileComponentsHolder    compHolder;

    // init
    BOOST_CHECK(eq.Init(params, loggerInstanceName, compHolder));
    BOOST_CHECK(eq.Start());

    // check
    ICommand* cmd = new SupportedMOTreeCommand;
    BOOST_CHECK(eq.Add(*cmd));

    // finalize
    BOOST_CHECK(eq.Stop());
    eq.Release();

    delete &eq;
}


BOOST_AUTO_TEST_CASE(AddUnSupportedCommand)
{
	LOG_DEBUG_(logger, "AddUnSupportedCommand test is running. ");

    IExecutionQueue& eq = *new ExecutionQueue;

    StringMap params;
    String loggerInstanceName = "ExecutionQueue";
    ProfileComponentsHolder    compHolder;

    // init
    BOOST_CHECK(eq.Init(params, loggerInstanceName, compHolder));
    BOOST_CHECK(eq.Start());

    // check
    ICommand* cmd = new UnsupportedCommand;
    bool cmdAdded = eq.Add(*cmd);
    BOOST_CHECK(!cmdAdded);
    if (!cmdAdded)
    {
        delete cmd;
    }

    // finalize
    BOOST_CHECK(eq.Stop());
    eq.Release();

    delete &eq;
}


